# This software is licensed by the MIT License (MIT), see LICENSE file
#               Copyright © 2024 Gregory Lirent <opensource@lirent.ru>

from django.db import models

from ..__main__ import OGClass, ProxyProperties
from ..page import OGSubpage

try:
    from django.conf import settings
    from django.utils.translation import gettext_lazy as _
except ImportError:
    settings = {}
    _ = lambda x: x

from .properties import OGPrice


@OGClass('og:price', ('price', 'amount'))
class OGGlobalPrice(OGPrice):

    class Meta:
        proxy = True


@OGClass('product', ('price', 'brand', 'availability', 'condition', 'gender', 'gtin', 'material', 'locale'))
@ProxyProperties('page', 'locale')
class OGProductPage(OGSubpage):
    """
    Information URI: https://developers.facebook.com/docs/marketing-api/catalog/reference/#og-tags
    _________________________________________

    Entities: ->Catalog

    og:price                   -> Current price of the item.
    product:brand              -> Brand name of the item.
    product:availability       -> Current availability of the item:
                                  * in stock
                                  * out of stock
                                  * available for order
                                  * discontinued
    product:condition          -> Current condition of the item: new, refurbished, used.
    product:gender             -> Determines gender for sizing:
                                  * female
                                  * male
                                  * unisex.
    product:gtin               -> 14 DIG Product's Global Trade Item Number (GTINs). Exclude dashes and spaces.
    material                   -> Material the item is made from. Supported values:
                                  * cotton
                                  * denim
                                  * leather
    product:locale             -> Specifies which website version the product is from; for example, en_GB for the UK website.
    product:price              ->
    product:mfr_part_no        ->
    product:catalog_id         ->
    product:mfr_part_no        ->
    product:sale_price         ->
    product:retailer_item_id   ->
    product:category           ->
    product:custom_label_[0-4] ->
    product:item_group_id      ->  see: Information URI


    _________________________________________
    """

    class Material:
        COTTON = 1
        DENIM = 2
        LEATHER = 3

    class Condition:
        NEW = 1
        REFURBISHED = 2
        USED = 3

    class Availability:
        IN_STOCK = 1
        OUT_OF_STOCK = 2
        AVAILABLE = 3
        DISCOUNTED = 4

    _materials = (
        (1, 'cotton'),
        (2, 'denim'),
        (3, 'leather')
    )

    _conditions = (
        (1, 'new'),
        (2, 'refurbished'),
        (3, 'used')
    )

    _availabilities = (
        (1, 'in stock'),
        (2, 'out of stock'),
        (3, 'available for order'),
        (4, 'discontinued')
    )

    class Meta:
        db_table = 'products'
        app_label = settings.get('OPENGRAPH_META_APP_LABEL', 'opengraph')
        verbose_name = _('product meta information')
        verbose_name_plural = _('products meta information')

    price = models.ForeignKey(OGGlobalPrice, on_delete=models.CASCADE, verbose_name=_('price'))
    availability = models.PositiveSmallIntegerField(_('availability'), choices=_availabilities, null=True)
    condition = models.PositiveSmallIntegerField(_('condition'), choices=_conditions, null=True)
    material = models.PositiveSmallIntegerField(_('material'), choices=_materials, null=True)
    brand = models.CharField(_('brand'), max_length=64, null=True)
    category = models.CharField(_('category'), max_length=255, null=True)
    gtin = models.CharField(_('gtin'), max_length=14, null=True)
    is_male = models.BooleanField(_('male'), null=True)

    @property
    def gender(self) -> str:
        return 'unisex' if self.is_male is None else 'male' if self.is_male else 'female'

    @gender.setter
    def gender(self, value) -> None:
        self.is_male = None if value not in ('male', 'female') else value == 'male'

    @gender.deleter
    def gender(self) -> None:
        self.is_male = None

    @classmethod
    def og_processing(cls, k, v, obj):
        if k.endswith((':availability', ':condition', ':material')):
            if k.endswith(':material'):
                k = 'material'
                choices = cls._materials
            else:
                choices = cls._availabilities if k.endswith(':availability') else cls._conditions
            for _k, _v in choices:
                if _k == v:
                    v = _v
                    break
            if not isinstance(v, str):
                raise AttributeError(f'unexpected {k} attribute value {v}')
        yield k, v

    def __init__(self, *args, **kwargs):
        kwargs['page_type'] = 'product'
        super().__init__(*args, **kwargs)
