# This software is licensed by the MIT License (MIT), see LICENSE file
#               Copyright © 2024 Gregory Lirent <opensource@lirent.ru>

from django.db import models

from ...__main__ import OGClass

try:
    from django.conf import settings
    from django.utils.translation import gettext_lazy as _
except ImportError:
    settings = {}
    _ = lambda x: x


@OGClass(':app', ('name', 'id', 'url'))
class TwitterCardAppInfo(models.Model):
    """
    URI: https://developers.facebook.com/blog/post/2013/06/05/platform-updates--new-open-graph-business-object-type/
    _________________________________________

    Entities:

    ...:app:name:... -> Name of The app
    ...:app:id:...   -> The app ID in the Google Play Store
    ...:app:url:...  -> The app's custom URL scheme

    _________________________________________
    """

    class Device:
        IPHONE = 1
        IPAD = 2
        ANDROID = 3

    _devices = (
        (1, 'iphone'),
        (2, 'ipad'),
        (3, 'googleplay')
    )

    class Meta:
        db_table = 'app'
        app_label = settings.get('TWITTER_CARD_META_APP_LABEL', 'twitter_card')
        verbose_name = _('app meta information')
        verbose_name_plural = _('apps meta information')

    type = models.PositiveSmallIntegerField(_('type'), choices=_devices)
    name = models.CharField(_('name'), max_length=127, null=True)
    id = models.CharField(_('id'), max_length=64, null=True)
    url = models.URLField(_('url'), max_length=255, null=True)

    @classmethod
    def og_processing(cls, k, v, obj):
        for _k, _v in cls._devices:
            if _k == obj.type:
                k = f'{k}:{_v}'
                break
        yield k, str(v)

    @classmethod
    def create_iphone_app(cls, *args, **kwargs):
        return TwitterCardIPhoneApp(*args, **kwargs)

    @classmethod
    def create_ipad_app(cls, *args, **kwargs):
        return TwitterCardIPadApp(*args, **kwargs)

    @classmethod
    def create_android_app(cls, *args, **kwargs):
        return TwitterCardAndroidApp(*args, **kwargs)

    def __init__(self, *args, **kwargs):
        if self.__class__ not in (TwitterCardIPhoneApp, TwitterCardIPadApp, TwitterCardAndroidApp):
            raise AttributeError('object creation is not allowed, '
                                 f'use {__class__.__name__}.create_[iphone | ipad | android]_app instead')
        super().__init__(*args, **kwargs)


class TwitterCardIPhoneApp(TwitterCardAppInfo):

    class Model:
        proxy = True

    def __init__(self, *args, **kwargs):
        kwargs['type'] = self.Device.IPHONE
        super().__init__(*args, **kwargs)


class TwitterCardIPadApp(TwitterCardAppInfo):

    class Model:
        proxy = True

    def __init__(self, *args, **kwargs):
        kwargs['type'] = self.Device.IPAD
        super().__init__(*args, **kwargs)


class TwitterCardAndroidApp(TwitterCardAppInfo):

    class Model:
        proxy = True

    def __init__(self, *args, **kwargs):
        kwargs['type'] = self.Device.ANDROID
        super().__init__(*args, **kwargs)
