# This software is licensed by the MIT License (MIT), see LICENSE file
#               Copyright © 2024 Gregory Lirent <opensource@lirent.ru>

from django.db import models

from ..__main__ import OGClass, ProxyProperties
from ..page import OGSubpage

try:
    from django.conf import settings
    from django.utils.translation import gettext_lazy as _
except ImportError:
    settings = {}
    _ = lambda x: x

from .properties import TwitterCardAppInfo, TwitterCardSite, TwitterCardCreator, TwitterCardPlayer
from ..properties import OGImage


class TwitterCard(OGSubpage):
    """
    Info URI: https://developer.twitter.com/en/docs/twitter-for-websites/cards/overview/markup
    _________________________________________

    twitter:card        -> The card type
    twitter:site        ->
    twitter:creator     -> see properties/profile.py
    twitter:description -> Description of content (maximum 200 characters)
    twitter:title       -> Title of content (max 70 characters)
    twitter:image       -> URL of image to use in the card. Images must be less than 5MB in size.
                           JPG, PNG, WEBP and GIF formats are supported.
                           Only the first frame of an animated GIF will be used. SVG is not supported.
    twitter:image:alt   -> A text description of the image conveying the essential nature of an image to users
                           who are visually impaired. Maximum 420 characters.
    twitter:player      -> see properties/player.py
    twitter:app         -> see properties/app.py
    _________________________________________
    """

    SUMMARY = 1
    SUMMARY_LARGE_IMAGE = 2
    PLAYER = 3
    APP = 4

    _cards = (
        (1, 'summary'),
        (2, 'summary_large_image'),
        (3, 'player'),
        (4, 'app')
    )

    class Meta:
        db_table = 'page'
        app_label = settings.get('TWITTER_CARD_META_APP_LABEL', 'twitter_card')
        verbose_name = _('twitter card information')
        verbose_name_plural = _('twitter cards information')

    card = models.PositiveSmallIntegerField(_('card types'), choices=_cards)
    site = models.ForeignKey(TwitterCardSite, verbose_name=_('site'), on_delete=models.CASCADE)

    @classmethod
    def create_player(cls, *args, **kwargs):
        return TwitterCardPlayer(*args, **kwargs)

    @classmethod
    def create_summary(cls, *args, **kwargs):
        return TwitterCardSummary(*args, **kwargs)

    @classmethod
    def create_summary_large_image(cls, *args, **kwargs):
        return TwitterCardSummaryLargeImage(*args, **kwargs)

    @classmethod
    def create_app(cls, *args, **kwargs):
        return TwitterCardApp(*args, **kwargs)

    @classmethod
    def og_processing(cls, k, v, obj):
        if k.endswith(':card'):
            for _k, _v in cls._cards:
                if _k == v:
                    v = _v
                    break
            if not isinstance(v, str):
                raise AttributeError(f'unexpected {k} attribute value {v}')
        yield k, v

    def __init__(self, *args, **kwargs):
        if self.__class__ not in (TwitterCardPlayer, TwitterCardSummary, TwitterCardSummaryLargeImage, TwitterCardApp):
            raise AttributeError('object creation is not allowed, '
                                 f'use {__class__.__name__}.create_[player | summary | summary_large_image | app] instead')
        super().__init__(*args, **kwargs)


@OGClass(':image', ('!url', 'alt'))
class TwitterCardImage(OGImage):

    class Meta:
        proxy = True


class TwitterMedia(TwitterCard):

    class Meta:
        proxy = True

    image = models.ForeignKey(TwitterCardImage, verbose_name=_('image'), on_delete=models.SET_NULL, null=True)


@OGClass('twitter', ('card', 'site', 'image', 'description', 'title'))
class TwitterCardPlayer(TwitterMedia):

    class Meta:
        proxy = True

    player = models.ForeignKey(TwitterCardPlayer, verbose_name=_('player'), on_delete=models.SET_NULL, null=True)

    def __init__(self, *args, **kwargs):
        kwargs['card'] = self.PLAYER
        super().__init__(*args, **kwargs)


@OGClass('twitter', ('card', 'site', 'image', 'creator', 'description', 'title'))
class TwitterCardSummary(TwitterMedia):

    class Meta:
        proxy = True

    creator = models.ForeignKey(TwitterCardCreator, verbose_name=_('creator'), on_delete=models.SET_NULL, null=True)

    def __init__(self, *args, **kwargs):
        if not kwargs['card']:
            kwargs['card'] = self.SUMMARY
        super().__init__(*args, **kwargs)


@OGClass('twitter', ('card', 'site', 'image', 'creator', 'description', 'title'))
class TwitterCardSummaryLargeImage(TwitterCardSummary):

    class Meta:
        proxy = True

    def __init__(self, *args, **kwargs):
        kwargs['card'] = self.SUMMARY_LARGE_IMAGE
        super().__init__(*args, **kwargs)


@OGClass('twitter', ('card', 'site', 'apps'))
class TwitterCardApp(TwitterCard):

    class Meta:
        proxy = True

    apps = models.ManyToManyField(TwitterCardAppInfo, verbose_name=_('apps'), null=True)

    def __init__(self, *args, **kwargs):
        kwargs['card'] = self.APP
        super().__init__(*args, **kwargs)
